# -*- coding: utf-8 -*-
#
# django-zodb - using Django and ZODB together
#
# Copyright (c) 2009, Triveos Tecnologia Ltda.
# See COPYING for license
#

from test_utils import URIParseTests
from test_storage import StorageTests
from test_database import DatabaseTests
from test_views import ViewsTests
from test_models import ModelsTests
from test_middleware import MiddlewareTests
